@@ -69,6 +69,7 @@ if 'DATABASE_URL' in os.environ:
     if url.scheme == 'mysql':
         DATABASES['default']['ENGINE'] = 'django.db.backends.mysql'
 
+EMAIL_SUBJECT_PREFIX = '[Sentry] '
 
 # Local time zone for this installation. Choices can be found here:
 # http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
@@ -110,6 +111,7 @@ MIDDLEWARE_CLASSES = (
     'django.middleware.csrf.CsrfViewMiddleware',
     'django.contrib.auth.middleware.AuthenticationMiddleware',
     'sentry.middleware.SentryMiddleware',
+    'sentry.middleware.SentrySocialAuthExceptionMiddleware',
     'django.middleware.locale.LocaleMiddleware',
     'django.contrib.messages.middleware.MessageMiddleware',
 )
@@ -150,16 +152,14 @@ INSTALLED_APPS = (
     'sentry',
     'sentry.plugins.sentry_interface_types',
     'sentry.plugins.sentry_mail',
-    'sentry.plugins.sentry_servers',
     'sentry.plugins.sentry_urls',
     'sentry.plugins.sentry_useragents',
     'social_auth',
     'south',
     'static_compiler',
-    'django_social_auth_trello',
 )
 
-STATIC_ROOT = os.path.join(PROJECT_ROOT, 'static')
+STATIC_ROOT = os.path.realpath(os.path.join(PROJECT_ROOT, 'static'))
 STATIC_URL = '/_static/'
 
 STATICFILES_FINDERS = (
@@ -186,29 +186,18 @@ else:
 AUTHENTICATION_BACKENDS = (
     'social_auth.backends.twitter.TwitterBackend',
     'social_auth.backends.facebook.FacebookBackend',
-    'social_auth.backends.google.GoogleOAuthBackend',
-    'social_auth.backends.google.GoogleOAuth2Backend',
+    # TODO: migrate to GoogleOAuth2Backend
     'social_auth.backends.google.GoogleBackend',
-    'social_auth.backends.yahoo.YahooBackend',
-    'social_auth.backends.browserid.BrowserIDBackend',
-    'social_auth.backends.contrib.linkedin.LinkedinBackend',
-    'social_auth.backends.contrib.livejournal.LiveJournalBackend',
-    'social_auth.backends.contrib.orkut.OrkutBackend',
-    'social_auth.backends.contrib.foursquare.FoursquareBackend',
     'social_auth.backends.contrib.github.GithubBackend',
-    'social_auth.backends.contrib.dropbox.DropboxBackend',
-    'social_auth.backends.contrib.flickr.FlickrBackend',
-    'social_auth.backends.contrib.instagram.InstagramBackend',
-    'social_auth.backends.contrib.skyrock.SkyrockBackend',
-    'social_auth.backends.contrib.yahoo.YahooOAuthBackend',
-    'social_auth.backends.OpenIDBackend',
     'social_auth.backends.contrib.bitbucket.BitbucketBackend',
-    'social_auth.backends.contrib.mixcloud.MixcloudBackend',
-    'social_auth.backends.contrib.live.LiveBackend',
-    'django_social_auth_trello.backend.TrelloBackend',
+    'social_auth.backends.contrib.trello.TrelloBackend',
     'sentry.utils.auth.EmailAuthBackend',
 )
 
+SOCIAL_AUTH_USER_MODEL = AUTH_USER_MODEL = 'auth.User'
+
+SESSION_ENGINE = "django.contrib.sessions.backends.signed_cookies"
+
 TWITTER_CONSUMER_KEY = ''
 TWITTER_CONSUMER_SECRET = ''
 
@@ -225,33 +214,66 @@ GITHUB_API_SECRET = ''
 TRELLO_API_KEY = ''
 TRELLO_API_SECRET = ''
 
+BITBUCKET_CONSUMER_KEY = ''
+BITBUCKET_CONSUMER_SECRET = ''
+
+SOCIAL_AUTH_PIPELINE = (
+    'social_auth.backends.pipeline.social.social_auth_user',
+    'social_auth.backends.pipeline.user.get_username',
+    'sentry.utils.social_auth.create_user_if_enabled',
+    'social_auth.backends.pipeline.social.associate_user',
+    'social_auth.backends.pipeline.social.load_extra_data',
+    'social_auth.backends.pipeline.user.update_user_details'
+)
+
 SOCIAL_AUTH_CREATE_USERS = True
 
+# Auth engines and the settings required for them to be listed
+AUTH_PROVIDERS = {
+    'twitter': ('TWITTER_CONSUMER_KEY', 'TWITTER_CONSUMER_SECRET'),
+    'facebook': ('FACEBOOK_APP_ID', 'FACEBOOK_API_SECRET'),
+    'github': ('GITHUB_APP_ID', 'GITHUB_API_SECRET'),
+    'google': ('GOOGLE_OAUTH2_CLIENT_ID', 'GOOGLE_OAUTH2_CLIENT_SECRET'),
+    'trello': ('TRELLO_API_KEY', 'TRELLO_API_SECRET'),
+    'bitbucket': ('BITBUCKET_CONSUMER_KEY', 'BITBUCKET_CONSUMER_SECRET'),
+}
+
 import random
 
 SOCIAL_AUTH_DEFAULT_USERNAME = lambda: random.choice(['Darth Vader', 'Obi-Wan Kenobi', 'R2-D2', 'C-3PO', 'Yoda'])
 SOCIAL_AUTH_PROTECTED_USER_FIELDS = ['email']
 
 # Queue configuration
+from kombu import Queue
 
 BROKER_URL = "django://"
 
+CELERY_ALWAYS_EAGER = True
 CELERY_IGNORE_RESULT = True
 CELERY_SEND_EVENTS = False
 CELERY_RESULT_BACKEND = None
 CELERY_TASK_RESULT_EXPIRES = 1
 CELERY_DISABLE_RATE_LIMITS = True
-
-# Sentry and Raven configuration
-
-SENTRY_PUBLIC = False
-SENTRY_PROJECT = 1
-SENTRY_CACHE_BACKEND = 'default'
-
-EMAIL_SUBJECT_PREFIX = '[Sentry] '
+CELERY_DEFAULT_QUEUE = "default"
+CELERY_DEFAULT_EXCHANGE = "default"
+CELERY_DEFAULT_EXCHANGE_TYPE = "direct"
+CELERY_DEFAULT_ROUTING_KEY = "default"
+CELERY_CREATE_MISSING_QUEUES = True
+CELERY_QUEUES = (
+    Queue('default', routing_key='default'),
+    Queue('celery', routing_key='celery'),
+    Queue('alerts', routing_key='alerts'),
+    Queue('cleanup', routing_key='cleanup'),
+    Queue('sourcemaps', routing_key='sourcemaps'),
+    Queue('search', routing_key='search'),
+    Queue('counters', routing_key='counters'),
+    Queue('events', routing_key='events'),
+    Queue('triggers', routing_key='triggers'),
+    Queue('update', routing_key='update'),
+)
 
 # Disable South in tests as it is sending incorrect create signals
-SOUTH_TESTS_MIGRATE = False
+SOUTH_TESTS_MIGRATE = True
 
 LOGGING = {
     'version': 1,
@@ -265,6 +287,11 @@ LOGGING = {
             'class': 'raven.contrib.django.handlers.SentryHandler',
         }
     },
+    'formatters': {
+        'client_info': {
+            'format': '%(name)s %(levelname)s %(project_slug)s/%(team_slug)s %(message)s'
+        }
+    },
     'loggers': {
         '()': {
             'handlers': ['console', 'sentry'],
@@ -277,6 +304,9 @@ LOGGING = {
             'handlers': ['console', 'sentry'],
             'propagate': False,
         },
+        'sentry.coreapi': {
+            'formatter': 'client_info',
+        },
         'sentry.errors': {
             'level': 'ERROR',
             'handlers': ['console'],
@@ -322,13 +352,34 @@ if os.path.exists(NPM_ROOT):
                     "sentry/scripts/lib/jquery.animate-colors.js",
                     "sentry/scripts/lib/jquery.clippy.min.js",
                     "sentry/scripts/lib/jquery.cookie.js",
-                    "sentry/scripts/lib/jquery.flot.min.js",
+                    "sentry/scripts/lib/jquery.flot.js",
+                    "sentry/scripts/lib/jquery.flot.resize.js",
+                    "sentry/scripts/lib/jquery.flot.stack.js",
+                    "sentry/scripts/lib/jquery.flot.time.js",
+                    "sentry/scripts/lib/jquery.flot.dashes.js",
                     "sentry/scripts/lib/simple-slider.js",
                     "sentry/scripts/lib/json2.js",
                     "sentry/scripts/lib/underscore.js",
                     "sentry/scripts/lib/backbone.js",
                     "sentry/scripts/lib/select2/select2.js",
-                    "sentry/scripts/lib/bootstrap.js",
+                ],
+            },
+            "sentry/scripts/bootstrap.min.js": {
+                "src": [
+                    "sentry/bootstrap/js/bootstrap-transition.js",
+                    "sentry/bootstrap/js/bootstrap-alert.js",
+                    "sentry/bootstrap/js/bootstrap-button.js",
+                    "sentry/bootstrap/js/bootstrap-carousel.js",
+                    "sentry/bootstrap/js/bootstrap-collapse.js",
+                    "sentry/bootstrap/js/bootstrap-dropdown.js",
+                    "sentry/bootstrap/js/bootstrap-modal.js",
+                    "sentry/bootstrap/js/bootstrap-tooltip.js",
+                    "sentry/bootstrap/js/bootstrap-popover.js",
+                    "sentry/bootstrap/js/bootstrap-scrollspy.js",
+                    "sentry/bootstrap/js/bootstrap-tab.js",
+                    "sentry/bootstrap/js/bootstrap-typeahead.js",
+                    "sentry/bootstrap/js/bootstrap-affix.js",
+                    "sentry/scripts/lib/bootstrap-datepicker.js"
                 ],
             },
             "sentry/styles/global.min.css": {
@@ -350,6 +401,116 @@ if os.path.exists(NPM_ROOT):
         },
     }
 
+# Sentry and Raven configuration
+
+SENTRY_PUBLIC = False
+SENTRY_PROJECT = 1
+SENTRY_CACHE_BACKEND = 'default'
+
+SENTRY_FILTERS = (
+    'sentry.filters.StatusFilter',
+)
+
+SENTRY_KEY = None
+
+# Absolute URL to the sentry root directory. Should not include a trailing slash.
+SENTRY_URL_PREFIX = ''
+
+# Allow access to Sentry without authentication.
+SENTRY_PUBLIC = False
+
+# Login url (defaults to LOGIN_URL)
+SENTRY_LOGIN_URL = None
+
+# Default project ID (for internal errors)
+SENTRY_PROJECT = 1
+
+# Only store a portion of all messages per unique group.
+SENTRY_SAMPLE_DATA = True
+
+# The following values control the sampling rates
+SENTRY_SAMPLE_RATES = (
+    (50, 1),
+    (1000, 2),
+    (10000, 10),
+    (100000, 50),
+    (1000000, 300),
+    (10000000, 2000),
+)
+SENTRY_MAX_SAMPLE_RATE = 10000
+SENTRY_SAMPLE_TIMES = (
+    (3600, 1),
+    (360, 10),
+    (60, 60),
+)
+SENTRY_MAX_SAMPLE_TIME = 10000
+
+# Web Service
+SENTRY_WEB_HOST = 'localhost'
+SENTRY_WEB_PORT = 9000
+SENTRY_WEB_OPTIONS = {
+    'workers': 3,
+    'limit_request_line': 0,  # required for raven-js
+}
+
+# UDP Service
+SENTRY_UDP_HOST = 'localhost'
+SENTRY_UDP_PORT = 9001
+
+# Queue (Kombu)
+SENTRY_QUEUE = {
+    'transport': 'kombu.transport.django.Transport',
+}
+
+# Should users without 'sentry.add_project' permissions be allowed
+# to create new projects
+SENTRY_ALLOW_PROJECT_CREATION = False
+
+# Should users without 'sentry.add_team' permissions be allowed
+# to create new projects
+SENTRY_ALLOW_TEAM_CREATION = False
+
+# Should users without superuser permissions be allowed to
+# make projects public
+SENTRY_ALLOW_PUBLIC_PROJECTS = True
+
+# Should users be allowed to register an account? If this is disabled
+# accounts can only be created when someone is invited or added
+# manually.
+SENTRY_ALLOW_REGISTRATION = True
+
+# Instructs Sentry to utilize it's internal search indexer on all incoming
+# events..
+SENTRY_USE_SEARCH = True
+
+# Enable trend results. These can be expensive and are calculated in real-time.
+# When disabled they will be replaced w/ a default priority sort.
+SENTRY_USE_TRENDING = True
+
+# Default to not sending the Access-Control-Allow-Origin header on api/store
+SENTRY_ALLOW_ORIGIN = None
+
+# Enable scraping of javascript context for source code
+SENTRY_SCRAPE_JAVASCRIPT_CONTEXT = True
+
+# Redis connection information (see Nydus documentation)
+SENTRY_REDIS_OPTIONS = {}
+
+# Buffer backend to use
+SENTRY_BUFFER = 'sentry.buffer.Buffer'
+SENTRY_BUFFER_OPTIONS = {}
+
+SENTRY_RAVEN_JS_URL = 'd3nslu0hdya83q.cloudfront.net/dist/1.0/raven.min.js'
+
+# URI Prefixes for generating DSN URLs
+# (Defaults to URL_PREFIX by default)
+SENTRY_ENDPOINT = None
+SENTRY_PUBLIC_ENDPOINT = None
+
+# Early draft features. Not slated or public release yet.
+SENTRY_ENABLE_EXPLORE_CODE = False
+SENTRY_ENABLE_EXPLORE_USERS = False
+
 # Configure celery
 import djcelery
 djcelery.setup_loader()
