@@ -24,6 +24,7 @@ from django.utils.translation import ugettext as _
 
 from sentry.app import env
 from sentry.models import UserOption
+from sentry.utils.strings import strip
 from sentry.web.helpers import render_to_string
 
 _Exception = Exception
@@ -40,7 +41,7 @@ def is_url(filename):
 
 
 def get_context(lineno, context_line, pre_context=None, post_context=None, filename=None,
-        format=False):
+                format=False):
     lineno = int(lineno)
     context = []
     start_lineno = lineno - len(pre_context or [])
@@ -88,6 +89,24 @@ def get_context(lineno, context_line, pre_context=None, post_context=None, filen
     return context
 
 
+def is_newest_frame_first(event):
+    newest_first = event.platform not in ('python', None)
+
+    if env.request and env.request.user.is_authenticated():
+        display = UserOption.objects.get_value(
+            user=env.request.user,
+            project=None,
+            key='stacktrace_order',
+            default=None,
+        )
+        if display == '1':
+            newest_first = False
+        elif display == '2':
+            newest_first = True
+
+    return newest_first
+
+
 class Interface(object):
     """
     An interface is a structured representation of data, which may
@@ -180,7 +199,7 @@ class Message(Interface):
     """
     attrs = ('message', 'params')
 
-    def __init__(self, message, params=()):
+    def __init__(self, message, params=(), **kwargs):
         self.message = message
         self.params = params
 
@@ -219,7 +238,7 @@ class Query(Interface):
     """
     attrs = ('query', 'engine')
 
-    def __init__(self, query, engine=None):
+    def __init__(self, query, engine=None, **kwargs):
         self.query = query
         self.engine = engine
 
@@ -242,9 +261,10 @@ class Frame(object):
     attrs = ('abs_path', 'filename', 'lineno', 'colno', 'in_app', 'context_line',
              'pre_context', 'post_context', 'vars', 'module', 'function', 'data')
 
-    def __init__(self, abs_path=None, filename=None, lineno=None, colno=None, in_app=False,
-                 context_line=None, pre_context=(), post_context=(), vars=None,
-                 module=None, function=None, data=None):
+    def __init__(self, abs_path=None, filename=None, lineno=None, colno=None,
+                 in_app=None, context_line=None, pre_context=(),
+                 post_context=(), vars=None, module=None, function=None,
+                 data=None, **kwargs):
         self.abs_path = abs_path or filename
         self.filename = filename or abs_path
 
@@ -284,6 +304,10 @@ class Frame(object):
     def is_valid(self):
         if self.in_app not in (False, True, None):
             return False
+        if type(self.vars) != dict:
+            return False
+        if type(self.data) != dict:
+            return False
         return self.filename or self.function or self.module
 
     def get_hash(self):
@@ -295,6 +319,11 @@ class Frame(object):
 
         if self.context_line is not None:
             output.append(self.context_line)
+        elif not output:
+            # If we were unable to achieve any context at this point
+            # (likely due to a bad JavaScript error) we should just
+            # bail on recording this frame
+            return output
         elif self.function:
             output.append(self.function)
         elif self.lineno is not None:
@@ -349,8 +378,10 @@ class Frame(object):
         else:
             choices = []
         choices.append('default')
-        templates = ['sentry/partial/frames/%s.txt' % choice
-                      for choice in choices]
+        templates = [
+            'sentry/partial/frames/%s.txt' % choice
+            for choice in choices
+        ]
         return render_to_string(templates, {
             'abs_path': self.abs_path,
             'filename': self.filename,
@@ -365,8 +396,8 @@ class Frame(object):
 class Stacktrace(Interface):
     """
     A stacktrace contains a list of frames, each with various bits (most optional)
-    describing the context of that frame. Frames should be sorted with the most recent
-    caller being the last in the list.
+    describing the context of that frame. Frames should be sorted from oldest
+    to newest.
 
     The stacktrace contains one element, ``frames``, which is a list of hashes. Each
     hash must contain **at least** the ``filename`` attribute. The rest of the values
@@ -432,13 +463,18 @@ class Stacktrace(Interface):
     >>>     }]
     >>> }
 
+    .. note:: This interface can be passed as the 'stacktrace' key in addition
+              to the full interface path.
     """
     attrs = ('frames',)
     score = 1000
 
-    def __init__(self, frames):
+    def __init__(self, frames, **kwargs):
         self.frames = [Frame(**f) for f in frames]
 
+    def __iter__(self):
+        return iter(self.frames)
+
     def validate(self):
         for frame in self.frames:
             # ensure we've got the correct required values
@@ -457,6 +493,9 @@ class Stacktrace(Interface):
             'frames': frames,
         }
 
+    def has_app_frames(self):
+        return any(f.in_app is not None for f in self.frames)
+
     def unserialize(self, data):
         data['frames'] = [Frame(**f) for f in data.pop('frames', [])]
         return data
@@ -464,11 +503,11 @@ class Stacktrace(Interface):
     def get_composite_hash(self, interfaces):
         output = self.get_hash()
         if 'sentry.interfaces.Exception' in interfaces:
-            exc = interfaces['sentry.interfaces.Exception']
+            exc = interfaces['sentry.interfaces.Exception'][0]
             if exc.type:
                 output.append(exc.type)
             elif not output:
-                output.append(exc.value)
+                output = exc.get_hash()
         return output
 
     def get_hash(self):
@@ -477,27 +516,8 @@ class Stacktrace(Interface):
             output.extend(frame.get_hash())
         return output
 
-    def is_newest_frame_first(self, event):
-        newest_first = event.platform not in ('python', None)
-
-        if env.request and env.request.user.is_authenticated():
-            display = UserOption.objects.get_value(
-                user=env.request.user,
-                project=None,
-                key='stacktrace_order',
-                default=None,
-            )
-            if display == '1':
-                newest_first = False
-            elif display == '2':
-                newest_first = True
-
-        return newest_first
-
-    def to_html(self, event, is_public=False, **kwargs):
-        if not self.frames:
-            return ''
-
+    def get_context(self, event, is_public=False, newest_first=None,
+                    with_stacktrace=True, **kwargs):
         system_frames = 0
         frames = []
         for frame in self.frames:
@@ -514,25 +534,36 @@ class Stacktrace(Interface):
             for frame in frames:
                 frame['in_app'] = True
 
-        newest_first = self.is_newest_frame_first(event)
+        if newest_first is None:
+            newest_first = is_newest_frame_first(event)
         if newest_first:
             frames = frames[::-1]
 
-        return render_to_string('sentry/partial/interfaces/stacktrace.html', {
+        context = {
             'is_public': is_public,
             'newest_first': newest_first,
             'system_frames': system_frames,
             'event': event,
             'frames': frames,
-            'stacktrace': self.get_traceback(event, newest_first=newest_first),
-        })
+            'stack_id': 'stacktrace_1',
+        }
+        if with_stacktrace:
+            context['stacktrace'] = self.get_traceback(event, newest_first=newest_first)
+        return context
+
+    def to_html(self, event, **kwargs):
+        context = self.get_context(
+            event=event,
+            **kwargs
+        )
+        return render_to_string('sentry/partial/interfaces/stacktrace.html', context)
 
     def to_string(self, event, is_public=False, **kwargs):
         return self.get_stacktrace(event, system_frames=False, max_frames=5)
 
     def get_stacktrace(self, event, system_frames=True, newest_first=None, max_frames=None):
         if newest_first is None:
-            newest_first = self.is_newest_frame_first(event)
+            newest_first = is_newest_frame_first(event)
 
         result = []
         if newest_first:
@@ -590,54 +621,100 @@ class Stacktrace(Interface):
         }
 
 
-class Exception(Interface):
+class SingleException(Interface):
     """
     A standard exception with a mandatory ``value`` argument, and optional
     ``type`` and``module`` argument describing the exception class type and
     module namespace.
 
+    You can also optionally bind a stacktrace interface to an exception. The
+    spec is identical to ``sentry.interfaces.Stacktrace``.
+
     >>>  {
     >>>     "type": "ValueError",
     >>>     "value": "My exception value",
     >>>     "module": "__builtins__"
+    >>>     "stacktrace": {
+    >>>         # see sentry.interfaces.Stacktrace
+    >>>     }
     >>> }
     """
-    attrs = ('value', 'type', 'module')
+    attrs = ('value', 'type', 'module', 'stacktrace')
 
     score = 900
     display_score = 1200
 
-    def __init__(self, value, type=None, module=None):
+    def __init__(self, value, type=None, module=None, stacktrace=None, **kwargs):
         # A human readable value for the exception
         self.value = value
         # The exception type name (e.g. TypeError)
         self.type = type
         # Optional module of the exception type (e.g. __builtin__)
         self.module = module
+        # Optional bound stacktrace interface
+        if stacktrace:
+            self.stacktrace = Stacktrace(**stacktrace)
+        else:
+            self.stacktrace = None
+
+    def validate(self):
+        if self.stacktrace:
+            return self.stacktrace.validate()
 
     def serialize(self):
+        if self.stacktrace:
+            stacktrace = self.stacktrace.serialize()
+        else:
+            stacktrace = None
+
         return {
-            'type': self.type,
-            'value': self.value,
-            'module': self.module,
+            'type': strip(self.type) or None,
+            'value': strip(self.value) or None,
+            'module': strip(self.module) or None,
+            'stacktrace': stacktrace,
         }
 
-    def get_hash(self):
-        return filter(bool, [self.type, self.value])
+    def unserialize(self, data):
+        if data.get('stacktrace'):
+            data['stacktrace'] = unserialize(Stacktrace, data['stacktrace'])
+        else:
+            data['stacktrace'] = None
+        return data
 
-    def to_html(self, event, is_public=False, **kwargs):
+    def get_hash(self):
+        output = None
+        if self.stacktrace:
+            output = self.stacktrace.get_hash()
+            if output and self.type:
+                output.append(self.type)
+        if not output:
+            output = filter(bool, [self.type, self.value])
+        return output
+
+    def get_context(self, event, is_public=False, **kwargs):
         last_frame = None
         interface = event.interfaces.get('sentry.interfaces.Stacktrace')
         if interface is not None and interface.frames:
             last_frame = interface.frames[-1]
-        return render_to_string('sentry/partial/interfaces/exception.html', {
+
+        e_module = strip(self.module)
+        e_type = strip(self.type) or 'Exception'
+        e_value = strip(self.value)
+
+        if self.module:
+            fullname = '%s.%s' % (e_module, e_type)
+        else:
+            fullname = e_type
+
+        return {
             'is_public': is_public,
             'event': event,
-            'exception_value': self.value,
-            'exception_type': self.type,
-            'exception_module': self.module,
+            'exception_value': e_value or e_type or '<empty value>',
+            'exception_type': e_type,
+            'exception_module': e_module,
+            'fullname': fullname,
             'last_frame': last_frame
-        })
+        }
 
     def get_search_context(self, event):
         return {
@@ -645,6 +722,134 @@ class Exception(Interface):
         }
 
 
+class Exception(Interface):
+    """
+    An exception consists of a list of values. In most cases, this list
+    contains a single exception, with an optional stacktrace interface.
+
+    Each exception has a mandatory ``value`` argument and optional ``type`` and
+    ``module`` arguments describing the exception class type and module
+    namespace.
+
+    You can also optionally bind a stacktrace interface to an exception. The
+    spec is identical to ``sentry.interfaces.Stacktrace``.
+
+    >>> [{
+    >>>     "type": "ValueError",
+    >>>     "value": "My exception value",
+    >>>     "module": "__builtins__"
+    >>>     "stacktrace": {
+    >>>         # see sentry.interfaces.Stacktrace
+    >>>     }
+    >>> }]
+
+    Values should be sent oldest to newest, this includes both the stacktrace
+    and the exception itself.
+
+    .. note:: This interface can be passed as the 'exception' key in addition
+              to the full interface path.
+    """
+
+    attrs = ('values',)
+    score = 2000
+
+    def __init__(self, *args, **kwargs):
+        if 'values' in kwargs:
+            values = kwargs['values']
+        elif not kwargs and len(args) == 1 and isinstance(args[0], (list, tuple)):
+            values = args[0]
+        else:
+            values = [kwargs]
+
+        self.values = [SingleException(**e) for e in values]
+
+    def __getitem__(self, key):
+        return self.values[key]
+
+    def __iter__(self):
+        return iter(self.values)
+
+    def __len__(self):
+        return len(self.values)
+
+    def validate(self):
+        for exception in self.values:
+            # ensure we've got the correct required values
+            exception.validate()
+
+    def serialize(self):
+        return {
+            'values': [e.serialize() for e in self.values]
+        }
+
+    def unserialize(self, data):
+        if 'values' not in data:
+            data = {'values': [data]}
+        data['values'] = [unserialize(SingleException, v) for v in data['values']]
+        return data
+
+    def get_hash(self):
+        return self.values[0].get_hash()
+
+    def get_composite_hash(self, interfaces):
+        return self.values[0].get_composite_hash(interfaces)
+
+    def get_context(self, event, is_public=False, **kwargs):
+        newest_first = is_newest_frame_first(event)
+        context_kwargs = {
+            'event': event,
+            'is_public': is_public,
+            'newest_first': newest_first,
+        }
+
+        exceptions = []
+        last = len(self.values) - 1
+        for num, e in enumerate(self.values):
+            context = e.get_context(**context_kwargs)
+            if e.stacktrace:
+                context['stacktrace'] = e.stacktrace.get_context(
+                    with_stacktrace=False, **context_kwargs)
+            else:
+                context['stacktrace'] = {}
+            context['stack_id'] = 'exception_%d' % (num,)
+            context['is_root'] = num == last
+            exceptions.append(context)
+
+        if newest_first:
+            exceptions.reverse()
+
+        return {
+            'newest_first': newest_first,
+            'system_frames': sum(e['stacktrace'].get('system_frames', 0) for e in exceptions),
+            'exceptions': exceptions,
+            'stacktrace': self.get_stacktrace(event, newest_first=newest_first)
+        }
+
+    def to_html(self, event, **kwargs):
+        if not self.values:
+            return ''
+
+        if len(self.values) == 1 and not self.values[0].stacktrace:
+            exception = self.values[0]
+            context = exception.get_context(event=event, **kwargs)
+            return render_to_string('sentry/partial/interfaces/exception.html', context)
+
+        context = self.get_context(event=event, **kwargs)
+        return render_to_string('sentry/partial/interfaces/chained_exception.html', context)
+
+    def to_string(self, event, is_public=False, **kwargs):
+        return self.get_stacktrace(event, system_frames=False, max_frames=5)
+
+    def get_search_context(self, event):
+        return self.values[0].get_search_context(event)
+
+    def get_stacktrace(self, *args, **kwargs):
+        exc = self.values[0]
+        if exc.stacktrace:
+            return exc.stacktrace.get_stacktrace(*args, **kwargs)
+        return ''
+
+
 class Http(Interface):
     """
     The Request information is stored in the Http interface. Two arguments
@@ -673,6 +878,9 @@ class Http(Interface):
     >>>         "REMOTE_ADDR": "192.168.0.1"
     >>>     }
     >>>  }
+
+    .. note:: This interface can be passed as the 'request' key in addition
+              to the full interface path.
     """
     attrs = ('url', 'method', 'data', 'query_string', 'cookies', 'headers',
              'env')
@@ -760,7 +968,8 @@ class Http(Interface):
         data = self.data
         headers_is_dict, headers = self._to_dict(self.headers)
 
-        if headers_is_dict and headers.get('Content-Type') == 'application/x-www-form-urlencoded':
+        # educated guess as to whether the body is normal POST data
+        if headers_is_dict and headers.get('Content-Type') == 'application/x-www-form-urlencoded' and '=' in data:
             _, data = self._to_dict(data)
 
         context = {
@@ -815,13 +1024,16 @@ class Template(Interface):
     >>>         "line5"
     >>>     ],
     >>> }
+
+    .. note:: This interface can be passed as the 'template' key in addition
+              to the full interface path.
     """
     attrs = ('filename', 'context_line', 'lineno', 'pre_context', 'post_context',
              'abs_path')
     score = 1100
 
     def __init__(self, filename, context_line, lineno, pre_context=None, post_context=None,
-                 abs_path=None):
+                 abs_path=None, **kwargs):
         self.abs_path = abs_path
         self.filename = filename
         self.context_line = context_line
