@@ -126,7 +126,7 @@
                         allowClear: true,
                         minimumInputLength: 3,
                         ajax: {
-                            url: this.getSearchTagsUrl(),
+                            url: app.utils.getSearchTagsUrl(),
                             dataType: 'json',
                             data: function (term, page) {
                                 return {
@@ -175,10 +175,6 @@
                 this.control.removeClass('realtime-play');
                 this.control.html(this.control.attr('data-play-label'));
             }
-        },
-
-        getSearchTagsUrl: function(){
-            return app.config.urlPrefix + '/api/' + app.config.teamId + '/' + app.config.projectId + '/tags/search/';
         }
 
     });
@@ -194,6 +190,59 @@
 
     });
 
+    app.SelectTeamPage = BasePage.extend({
+
+        initialize: function(){
+            BasePage.prototype.initialize.call(this);
+
+            this.refreshSparklines();
+            $(window).on('resize', this.refreshSparklines);
+        },
+
+        refreshSparklines: function(){
+            $('.chart').each(function(n, el){
+                var $el = $(el);
+                $.ajax({
+                    url: $el.attr('data-api-url'),
+                    type: 'get',
+                    dataType: 'json',
+                    data: {
+                        days: 1
+                    },
+                    success: _.bind(function(data){
+                        $.plot($el, [{
+                                data: data,
+                                color: '#ebeff3',
+                                shadowSize: 0,
+                                lines: {
+                                    lineWidth: 2,
+                                    show: true,
+                                    fill: true,
+                                    color: '#f6f8fa'
+                            }
+                            }], {
+                                yaxis: {
+                                    min: 0
+                                },
+                                grid: {
+                                    show: false
+                                },
+                                hoverable: false,
+                                legend: {
+                                    noColumns: 5
+                                },
+                                lines: {
+                                    show: false
+                                }
+                            }
+                        );
+                    }, this)
+                });
+            });
+        }
+
+    });
+
     app.GroupDetailsPage = BasePage.extend({
 
         initialize: function(data){
@@ -308,7 +357,6 @@
     });
 
     app.WallPage = BasePage.extend({
-
         initialize: function(){
             BasePage.prototype.initialize.call(this, {
                 realtime: true,
@@ -323,6 +371,17 @@
             this.refreshStats();
         },
 
+        makeDefaultView: function(id){
+            return new app.GroupListView({
+                className: 'group-list',
+                id: id,
+                maxItems: 5,
+                stream: this.options.stream,
+                realtime: this.options.realtime,
+                model: app.models.Group
+            });
+        },
+
         refreshSparkline: function(){
             $.ajax({
                 url: this.sparkline.attr('data-api-url'),
@@ -370,7 +429,43 @@
                 success: _.bind(function(data){
                     this.stats.find('[data-stat]').each(function(){
                         var $this = $(this);
-                        $this.find('big').text(data[$this.attr('data-stat')]);
+                        var new_count = data[$this.attr('data-stat')];
+                        var counter = $this.find('big');
+                        var digit = counter.find('span');
+
+                        if (digit.is(':animated'))
+                            return false;
+
+                        if (counter.data('count') == new_count) {
+                            // We are already showing this number
+                            return false;
+                        }
+
+                        counter.data('count', new_count);
+
+                        var replacement = $('<span></span>', {
+                            css: {
+                                top: '-2.1em',
+                                opacity: 0
+                            },
+                            text: new_count
+                        });
+
+                        // The .static class is added when the animation
+                        // completes. This makes it run smoother.
+
+                        digit.before(replacement).animate({
+                            top: '2.5em',
+                            opacity: 0
+                        }, 'fast', function(){
+                            digit.remove();
+                        });
+
+                        replacement.delay(100).animate({
+                            top: 0,
+                            opacity: 1
+                        }, 'fast');
+
                     });
                     window.setTimeout(this.refreshStats, 1000);
                 }, this)
@@ -394,11 +489,6 @@
     });
 
     app.AddTeamMemberPage = BasePage.extend({
-        initialize: function(data){
-            BasePage.prototype.initialize.call(this, data);
-
-            app.utils.makeSearchableUsersInput('form input[name=add-user]');
-        }
     });
 
     app.AccessGroupMembersPage = BasePage.extend({
@@ -481,6 +571,44 @@
         }
     });
 
+    app.ProjectNotificationsPage = BasePage.extend({
+        initialize: function initialize(data){
+            BasePage.prototype.initialize.call(this, data);
+
+            $("input[type=range]").each(_.bind(function loop(n, el){
+                var $el = $(el),
+                    min = parseInt($el.attr('min'), 10),
+                    max = parseInt($el.attr('max'), 10),
+                    step = parseInt($el.attr('step'), 10),
+                    $value = $('<span class="value"></span>');
+
+                $el.on("slider:ready", _.bind(function sliderready(event, data) {
+                    $value.appendTo(data.el);
+                    $value.html(this.formatThreshold(data.value));
+                }, this)).on("slider:changed", _.bind(function sliderchanged(event, data) {
+                    $value.html(this.formatThreshold(data.value));
+                }, this)).simpleSlider({
+                    range: [min, max],
+                    step: step,
+                    snap: true
+                });
+            }, this));
+
+            $("#tag_list input").each(function(_, el){
+                $(el).addClass('span6');
+                app.utils.makeSearchableTagsInput(el);
+            });
+        },
+
+        formatThreshold: function formatThreshold(value) {
+            if (!value) {
+                return 'Disabled';
+            }
+            return value + '%';
+        }
+
+    });
+
     app.NewProjectPage = BasePage.extend({
 
         initialize: function(data){
@@ -506,4 +634,4 @@
     Backbone.sync = function(method, model, success, error){
         success();
     };
-}(window, app, Backbone, jQuery, _));
\ No newline at end of file
+}(window, app, Backbone, jQuery, _));
