@@ -19,6 +19,9 @@
 
         render: function(){
             var data = this.model.toJSON();
+            data.loggerUrl = app.config.urlPrefix + '/' + app.config.teamId +
+                '/' + app.config.projectId + '/?logger=' + data.logger;
+
             this.$el.html(this.template(data));
             this.$el.attr('data-id', this.model.id);
             this.$el.addClass(this.getLevelClassName());
@@ -62,6 +65,7 @@
                 type: 'post',
                 dataType: 'json',
                 success: _.bind(function(response) {
+                    this.model.set('version', response.version);
                     this.model.set('isResolved', true);
                 }, this)
             });
@@ -73,6 +77,7 @@
                 type: 'post',
                 dataType: 'json',
                 success: _.bind(function(response) {
+                    this.model.set('version', response.version);
                     this.model.set('isResolved', false);
                 }, this)
             });
@@ -103,6 +108,7 @@
                     gid: this.model.get('id')
                 },
                 success: _.bind(function(response){
+                    this.model.set('version', response.version);
                     this.model.set('isBookmarked', response.isBookmarked);
                 }, this)
             });
@@ -204,6 +210,12 @@
                 this.setEmpty();
                 this.loaded = false;
             } else {
+                var _members = [];
+                _.each(members, _.bind(function(m){
+                    _members.push(new this.model(m));
+                }, this));
+                members = _members;
+
                 this.$empty.html(this.emptyMessage);
                 this.collection.reset(members);
                 this.loaded = true;
@@ -250,11 +262,7 @@
                 options = {};
 
             var existing = this.collection.get(member.id);
-            for (var key in member.attributes) {
-                if (existing.get(key) != member.get(key)) {
-                    existing.set(key, member.get(key));
-                }
-            }
+            existing.updateFrom(member);
 
             if (options.sort !== false) {
                 // score changed, resort
