@@ -6,13 +6,13 @@ sentry.tasks.fetch_source
 :license: BSD, see LICENSE for more details.
 """
 
-import zlib
+import logging
 import hashlib
 import urllib2
+import zlib
 from collections import namedtuple
 from urlparse import urljoin
 
-from celery.task import task
 from django.utils.simplejson import JSONDecodeError
 from sentry.utils.cache import cache
 from sentry.utils.sourcemaps import sourcemap_to_index, find_source
@@ -22,9 +22,10 @@ BAD_SOURCE = -1
 # number of surrounding lines (on each side) to fetch
 LINES_OF_CONTEXT = 5
 
-
 UrlResult = namedtuple('UrlResult', ['url', 'headers', 'body'])
 
+logger = logging.getLogger(__name__)
+
 
 def trim_line(line):
     line = line.strip('\n')
@@ -92,19 +93,12 @@ def discover_sourcemap(result, logger=None):
     return sourcemap
 
 
-def fetch_url(url, logger=None):
+def fetch_url_content(url, logger=None):
     """
-    Pull down a URL, returning a UrlResult object.
-
-    Attempts to fetch from the cache.
+    Pull down a URL, returning a tuple (url, headers, body).
     """
     import sentry
 
-    cache_key = 'fetch_url:%s' % (hashlib.md5(url).hexdigest(),)
-    result = cache.get(cache_key)
-    if result is not None:
-        return result
-
     try:
         opener = urllib2.build_opener()
         opener.addheaders = [('User-Agent', 'Sentry/%s' % sentry.VERSION)]
@@ -122,11 +116,28 @@ def fetch_url(url, logger=None):
             logger.error('Unable to fetch remote source for %r', url, exc_info=True)
         return BAD_SOURCE
 
-    result = UrlResult(url, headers, body)
+    return (url, headers, body)
 
-    cache.set(cache_key, result, 60 * 5)
 
-    return result
+def fetch_url(url, logger=None):
+    """
+    Pull down a URL, returning a UrlResult object.
+
+    Attempts to fetch from the cache.
+    """
+
+    cache_key = 'fetch_url:v2:%s' % (
+        hashlib.md5(url.encode('utf-8')).hexdigest(),)
+    result = cache.get(cache_key)
+    if result is None:
+        result = fetch_url_content(url, logger)
+
+        cache.set(cache_key, result, 60 * 5)
+
+    if result == BAD_SOURCE:
+        return result
+
+    return UrlResult(*result)
 
 
 def fetch_sourcemap(url, logger=None):
@@ -142,16 +153,14 @@ def fetch_sourcemap(url, logger=None):
         body = body.split('\n', 1)[1]
     try:
         index = sourcemap_to_index(body)
-    except JSONDecodeError:
+    except (JSONDecodeError, ValueError):
         if logger:
-            logger.warning('Failed parsing sourcemap JSON: %r', body[:15],
-            exc_info=True)
+            logger.warning('Failed parsing sourcemap JSON: %r', body[:15], exc_info=True)
     else:
         return index
 
 
-@task(ignore_result=True)
-def fetch_javascript_source(event, **kwargs):
+def expand_javascript_source(data, **kwargs):
     """
     Attempt to fetch source code for javascript frames.
 
@@ -161,23 +170,27 @@ def fetch_javascript_source(event, **kwargs):
     - colno >= 0
     - abs_path is the HTTP URI to the source
     - context_line is empty
+
+    Mutates the input ``data`` with expanded context if available.
     """
-    logger = fetch_javascript_source.get_logger()
+    from sentry.interfaces import Stacktrace
 
     try:
-        stacktrace = event.interfaces['sentry.interfaces.Stacktrace']
+        stacktrace = Stacktrace(**data['sentry.interfaces.Stacktrace'])
     except KeyError:
-        logger.debug('No stacktrace for event %r', event.id)
+        logger.debug('No stacktrace for event %r', data['event_id'])
         return
 
     # build list of frames that we can actually grab source for
-    frames = [f for f in stacktrace.frames
+    frames = [
+        f for f in stacktrace.frames
         if f.lineno is not None
-            and f.context_line is None
-            and f.is_url()]
+        and f.is_url()
+    ]
+
     if not frames:
-        logger.debug('Event %r has no frames with enough context to fetch remote source', event.id)
-        return
+        logger.debug('Event %r has no frames with enough context to fetch remote source', data['event_id'])
+        return data
 
     file_list = set()
     sourcemap_capable = set()
@@ -266,5 +279,4 @@ def fetch_javascript_source(event, **kwargs):
             source=source, lineno=frame.lineno)
 
     if has_changes:
-        event.data['sentry.interfaces.Stacktrace'] = stacktrace.serialize()
-        event.update(data=event.data)
+        data['sentry.interfaces.Stacktrace'] = stacktrace.serialize()
