@@ -8,12 +8,10 @@ sentry.web.forms.accounts
 
 from django import forms
 from django.contrib.auth import authenticate
-from django.contrib.auth.models import User
 from django.utils.translation import ugettext_lazy as _
 
-from sentry.conf import settings
-from sentry.constants import EMPTY_PASSWORD_VALUES
-from sentry.models import UserOption
+from sentry.constants import EMPTY_PASSWORD_VALUES, LANGUAGES
+from sentry.models import UserOption, User
 
 
 class RegistrationForm(forms.ModelForm):
@@ -43,7 +41,11 @@ class RegistrationForm(forms.ModelForm):
 
 
 class NotificationSettingsForm(forms.Form):
-    alert_email = forms.EmailField(help_text=_('Designate an alternative email address to send email notifications to.'))
+    alert_email = forms.EmailField(help_text=_('Designate an alternative email address to send email notifications to.'), required=False)
+    subscribe_by_default = forms.ChoiceField(choices=(
+        (1, _('Automatically subscribe to notifications for new projects')),
+        (0, _('Do not subscribe to notifications for new projects')),
+    ), required=False)
 
     def __init__(self, user, *args, **kwargs):
         self.user = user
@@ -54,6 +56,12 @@ class NotificationSettingsForm(forms.Form):
             key='alert_email',
             default=user.email,
         )
+        self.fields['subscribe_by_default'].initial = UserOption.objects.get_value(
+            user=self.user,
+            project=None,
+            key='subscribe_by_default',
+            default=1,
+        )
 
     def get_title(self):
         return "General"
@@ -65,6 +73,12 @@ class NotificationSettingsForm(forms.Form):
             key='alert_email',
             value=self.cleaned_data['alert_email'],
         )
+        UserOption.objects.set_value(
+            user=self.user,
+            project=None,
+            key='subscribe_by_default',
+            value=self.cleaned_data['subscribe_by_default'],
+        )
 
 
 class AccountSettingsForm(forms.Form):
@@ -102,7 +116,7 @@ class AccountSettingsForm(forms.Form):
 
 
 class AppearanceSettingsForm(forms.Form):
-    language = forms.ChoiceField(label=_('Language'), choices=settings.LANGUAGES, required=False)
+    language = forms.ChoiceField(label=_('Language'), choices=LANGUAGES, required=False)
     stacktrace_order = forms.ChoiceField(label=_('Stacktrace order'), choices=(
         ('-1', _('Default (let Sentry decide)')),
         ('1', _('Most recent call last')),
@@ -155,3 +169,40 @@ class RecoverPasswordForm(forms.Form):
 
 class ChangePasswordRecoverForm(forms.Form):
     password = forms.CharField(widget=forms.PasswordInput())
+
+
+class ProjectEmailOptionsForm(forms.Form):
+    alert = forms.BooleanField(required=False)
+    email = forms.EmailField(required=False, widget=forms.HiddenInput())
+
+    def __init__(self, project, user, *args, **kwargs):
+        self.project = project
+        self.user = user
+
+        super(ProjectEmailOptionsForm, self).__init__(*args, **kwargs)
+
+        is_enabled = UserOption.objects.get_value(
+            user, project, 'mail:alert', None)
+        if is_enabled is None:
+            is_enabled = UserOption.objects.get_value(
+                user, None, 'subscribe_by_default', 1) == 1
+        else:
+            is_enabled = bool(is_enabled)
+
+        self.fields['alert'].initial = is_enabled
+        self.fields['email'].initial = UserOption.objects.get_value(
+            user, project, 'mail:email', None)
+
+    def save(self):
+        UserOption.objects.set_value(
+            self.user, self.project, 'mail:alert',
+            int(self.cleaned_data['alert']),
+        )
+        if self.cleaned_data['email']:
+            UserOption.objects.set_value(
+                self.user, self.project, 'mail:email',
+                self.cleaned_data['email'],
+            )
+        else:
+            UserOption.objects.unset_value(
+                self.user, self.project, 'mail:email')
