@@ -13,16 +13,16 @@ from django.views.decorators.csrf import csrf_protect
 from django.views.decorators.http import require_http_methods
 from django.utils.translation import ugettext_lazy as _
 
-from sentry.constants import MEMBER_OWNER
-from sentry.models import Project, ProjectKey, Team, FilterKey
-from sentry.permissions import (can_create_projects, can_remove_project, can_create_teams,
-    can_add_project_key, can_remove_project_key)
+from sentry.constants import (
+    MEMBER_OWNER, STATUS_HIDDEN, DEFAULT_ALERT_PROJECT_THRESHOLD)
+from sentry.models import Project, ProjectKey, Team, TagKey
+from sentry.permissions import (
+    can_remove_project, can_add_project_key, can_remove_project_key)
 from sentry.plugins import plugins
 from sentry.web.decorators import login_required, has_access
-from sentry.web.forms.projects import (NewProjectForm, NewProjectAdminForm,
-    ProjectTagsForm, EditProjectForm, RemoveProjectForm, EditProjectAdminForm,
-    NotificationTagValuesForm)
-from sentry.web.forms.teams import NewTeamForm, SelectTeamForm
+from sentry.web.forms.projects import (
+    ProjectTagsForm, RemoveProjectForm, EditProjectForm,
+    NotificationTagValuesForm, AlertSettingsForm)
 from sentry.web.helpers import render_to_response, plugin_config
 
 
@@ -32,74 +32,6 @@ def get_started(request, team, project):
     return render_to_response('sentry/get_started.html', {
         'project': project,
         'team': project.team,
-        'SECTION': 'team',
-        'SUBSECTION': 'projects'
-    }, request)
-
-
-# TODO: we need a team specific project creation view, vs the "get started" view
-@login_required
-def new_project(request):
-    from django.contrib.auth.models import User
-
-    if not can_create_projects(request.user):
-        return HttpResponseRedirect(reverse('sentry'))
-
-    allow_create_teams = can_create_teams(request.user)
-    team_list = Team.objects.get_for_user(request.user)
-
-    if request.user.has_perm('sentry.can_add_project') and User.objects.all()[0:2] == 2:
-        project_form_cls = NewProjectAdminForm
-        project_initial = {
-            'owner': request.user.username,
-        }
-    else:
-        project_form_cls = NewProjectForm
-        project_initial = {}
-
-    if len(team_list) > 0:
-        select_team_form = SelectTeamForm(team_list, request.POST or None, prefix='st')
-    elif not allow_create_teams:
-        return render_to_response('sentry/projects/cannot_create_teams.html', {}, request)
-    else:
-        select_team_form = None
-
-    if allow_create_teams:
-        new_team_form = NewTeamForm(request.POST or None, prefix='nt')
-    else:
-        new_team_form = None
-
-    project_form = project_form_cls(request.POST or None, initial=project_initial, prefix='prj')
-
-    is_new_team = new_team_form and new_team_form.is_valid()
-    if is_new_team or not select_team_form:
-        team_form = new_team_form
-    else:
-        team_form = select_team_form
-
-    if project_form.is_valid() and team_form.is_valid():
-        project = project_form.save(commit=False)
-        if not project.owner:
-            project.owner = request.user
-
-        if is_new_team:
-            team = new_team_form.save(commit=False)
-            team.owner = project.owner
-            team.save()
-        else:
-            team = select_team_form.cleaned_data['team']
-
-        project.team = team
-        project.save()
-
-        if project.platform not in (None, 'other'):
-            return HttpResponseRedirect(reverse('sentry-docs-client', args=[project.team.slug, project.slug, project.platform]))
-        return HttpResponseRedirect(reverse('sentry-get-started', args=[project.team.slug, project.slug]))
-
-    return render_to_response('sentry/projects/new.html', {
-        'project_form': project_form,
-        'select_team_form': select_team_form,
-        'new_team_form': new_team_form,
     }, request)
 
 
@@ -116,12 +48,18 @@ def remove_project(request, team, project):
     if form.is_valid():
         removal_type = form.cleaned_data['removal_type']
         if removal_type == '1':
-            project.delete()
+            from sentry.tasks.deletion import delete_project
+
+            delete_project.delay(object_id=project.id)
+            project.update(status=STATUS_HIDDEN)
+
+            messages.add_message(request, messages.SUCCESS,
+                _('Deletion has been queued and should occur shortly.'))
         elif removal_type == '2':
             new_project = form.cleaned_data['project']
             project.merge_to(new_project)
         elif removal_type == '3':
-            project.update(status=1)
+            project.update(status=STATUS_HIDDEN)
         else:
             raise ValueError(removal_type)
 
@@ -132,8 +70,6 @@ def remove_project(request, team, project):
         'team': team,
         'form': form,
         'project': project,
-        'SECTION': 'team',
-        'SUBSECTION': 'projects'
     })
 
     return render_to_response('sentry/projects/remove.html', context, request)
@@ -146,17 +82,9 @@ def manage_project(request, team, project):
     if result is False and not request.user.has_perm('sentry.can_change_project'):
         return HttpResponseRedirect(reverse('sentry'))
 
-    # XXX: We probably shouldn't allow changing the team unless they're the project owner
     team_list = Team.objects.get_for_user(project.owner or request.user, MEMBER_OWNER)
 
-    can_admin_project = request.user == project.owner or request.user.has_perm('sentry.can_change_project')
-
-    if can_admin_project:
-        form_cls = EditProjectAdminForm
-    else:
-        form_cls = EditProjectForm
-
-    form = form_cls(request, team_list, request.POST or None, instance=project, initial={
+    form = EditProjectForm(request, team_list, request.POST or None, instance=project, initial={
         'origins': '\n'.join(project.get_option('sentry:origins', None) or []),
         'owner': project.owner,
         'resolve_age': int(project.get_option('sentry:resolve_age', 0)),
@@ -166,7 +94,8 @@ def manage_project(request, team, project):
         project = form.save()
         project.update_option('sentry:origins', form.cleaned_data.get('origins') or [])
         project.update_option('sentry:resolve_age', form.cleaned_data.get('resolve_age'))
-        messages.add_message(request, messages.SUCCESS,
+        messages.add_message(
+            request, messages.SUCCESS,
             _('Changes to your project were saved.'))
 
         return HttpResponseRedirect(reverse('sentry-manage-project', args=[team.slug, project.slug]))
@@ -178,8 +107,6 @@ def manage_project(request, team, project):
         'page': 'details',
         'form': form,
         'project': project,
-        'SECTION': 'team',
-        'SUBSECTION': 'projects'
     })
 
     return render_to_response('sentry/projects/manage.html', context, request)
@@ -207,8 +134,6 @@ def manage_project_keys(request, team, project):
         'project': project,
         'key_list': key_list,
         'can_add_key': can_add_project_key(request.user, project),
-        'SECTION': 'team',
-        'SUBSECTION': 'projects'
     })
 
     return render_to_response('sentry/projects/keys.html', context, request)
@@ -241,7 +166,8 @@ def remove_project_key(request, team, project, key_id):
         return HttpResponseRedirect(reverse('sentry-manage-project-keys', args=[project.team.slug, project.slug]))
 
     key.delete()
-    messages.add_message(request, messages.SUCCESS,
+    messages.add_message(
+        request, messages.SUCCESS,
         _('The API key (%s) was revoked.') % (key.public_key,))
 
     return HttpResponseRedirect(reverse('sentry-manage-project-keys', args=[project.team.slug, project.slug]))
@@ -249,7 +175,10 @@ def remove_project_key(request, team, project, key_id):
 
 @has_access(MEMBER_OWNER)
 def manage_project_tags(request, team, project):
-    tag_list = FilterKey.objects.all_keys(project)
+    tag_list = filter(
+        lambda x: not x.startswith('sentry:'),
+        TagKey.objects.all_keys(project))
+
     if tag_list:
         form = ProjectTagsForm(project, tag_list, request.POST or None)
     else:
@@ -258,7 +187,8 @@ def manage_project_tags(request, team, project):
     if form and form.is_valid():
         form.save()
 
-        messages.add_message(request, messages.SUCCESS,
+        messages.add_message(
+            request, messages.SUCCESS,
             _('Your settings were saved successfully.'))
 
         return HttpResponseRedirect(reverse('sentry-manage-project-tags', args=[project.team.slug, project.slug]))
@@ -269,8 +199,6 @@ def manage_project_tags(request, team, project):
         'page': 'tags',
         'project': project,
         'form': form,
-        'SECTION': 'team',
-        'SUBSECTION': 'projects'
     }
     return render_to_response('sentry/projects/manage_tags.html', context, request)
 
@@ -281,7 +209,7 @@ def notification_settings(request, team, project):
     initial = project.get_option('notifcation:tags', {})
 
     tag_forms = []
-    for tag in FilterKey.objects.all_keys(project):
+    for tag in TagKey.objects.all_keys(project):
         tag_forms.append(NotificationTagValuesForm(
             project=project,
             tag=tag,
@@ -292,7 +220,19 @@ def notification_settings(request, team, project):
             },
         ))
 
-    if request.method == 'POST' and all(f.is_valid() for f in tag_forms):
+    threshold, min_events = project.get_option(
+        'alert:threshold', DEFAULT_ALERT_PROJECT_THRESHOLD)
+
+    alert_form = AlertSettingsForm(
+        data=request.POST or None,
+        prefix='alert',
+        initial={
+            'pct_threshold': threshold,
+            'min_events': min_events,
+        }
+    )
+
+    if request.method == 'POST' and all(f.is_valid() for f in tag_forms) and alert_form.is_valid():
         tags = {}
         for form in tag_forms:
             values = form.cleaned_data['values']
@@ -300,7 +240,11 @@ def notification_settings(request, team, project):
                 tags[form.tag] = values
         project.update_option('notifcation:tags', tags)
 
-        messages.add_message(request, messages.SUCCESS,
+        project.update_option('alert:threshold', (
+            alert_form.cleaned_data['pct_threshold'], alert_form.cleaned_data['min_events']))
+
+        messages.add_message(
+            request, messages.SUCCESS,
             _('Your settings were saved successfully.'))
 
         return HttpResponseRedirect(reverse('sentry-project-notifications', args=[project.team.slug, project.slug]))
@@ -309,10 +253,9 @@ def notification_settings(request, team, project):
     context.update({
         'team': team,
         'project': project,
+        'alert_form': alert_form,
         'tag_forms': tag_forms,
         'page': 'notifications',
-        'SECTION': 'team',
-        'SUBSECTION': 'projects',
     })
     return render_to_response('sentry/projects/notifications.html', context, request)
 
@@ -330,7 +273,8 @@ def manage_plugins(request, team, project):
             if plugin.can_enable_for_projects():
                 plugin.set_option('enabled', plugin.slug in enabled, project)
 
-        messages.add_message(request, messages.SUCCESS,
+        messages.add_message(
+            request, messages.SUCCESS,
             _('Your settings were saved successfully.'))
 
         return HttpResponseRedirect(request.path)
@@ -340,8 +284,6 @@ def manage_plugins(request, team, project):
         'team': team,
         'page': 'plugins',
         'project': project,
-        'SECTION': 'team',
-        'SUBSECTION': 'projects'
     })
 
     return render_to_response('sentry/projects/plugins/list.html', context, request)
@@ -368,7 +310,8 @@ def configure_project_plugin(request, team, project, slug):
 
     action, view = plugin_config(plugin, project, request)
     if action == 'redirect':
-        messages.add_message(request, messages.SUCCESS,
+        messages.add_message(
+            request, messages.SUCCESS,
             _('Your settings were saved successfully.'))
 
         return HttpResponseRedirect(request.path)
@@ -382,8 +325,6 @@ def configure_project_plugin(request, team, project, slug):
         'project': project,
         'plugin': plugin,
         'plugin_is_enabled': plugin.is_enabled(project),
-        'SECTION': 'team',
-        'SUBSECTION': 'projects'
     })
 
     return render_to_response('sentry/projects/plugins/configure.html', context, request)
@@ -415,10 +356,12 @@ def enable_project_plugin(request, team, project, slug):
     try:
         plugin = plugins.get(slug)
     except KeyError:
-        return HttpResponseRedirect(reverse('sentry-configure-project-plugin', args=[project.team.slug, project.slug, slug]))
+        return HttpResponseRedirect(reverse('sentry-manage-project-plugins', args=[project.team.slug, project.slug]))
+
+    redirect_to = reverse('sentry-configure-project-plugin', args=[project.team.slug, project.slug, slug])
 
     if plugin.is_enabled(project) or not plugin.can_enable_for_projects():
-        return HttpResponseRedirect(reverse('sentry-configure-project-plugin', args=[project.team.slug, project.slug, slug]))
+        return HttpResponseRedirect(redirect_to)
 
     result = plugins.first('has_perm', request.user, 'configure_project_plugin', project, plugin)
     if result is False and not request.user.is_superuser:
@@ -426,7 +369,7 @@ def enable_project_plugin(request, team, project, slug):
 
     plugin.set_option('enabled', True, project)
 
-    return HttpResponseRedirect(reverse('sentry-configure-project-plugin', args=[project.team.slug, project.slug, slug]))
+    return HttpResponseRedirect(redirect_to)
 
 
 @has_access(MEMBER_OWNER)
@@ -435,10 +378,12 @@ def disable_project_plugin(request, team, project, slug):
     try:
         plugin = plugins.get(slug)
     except KeyError:
-        return HttpResponseRedirect(reverse('sentry-configure-project-plugin', args=[project.team.slug, project.slug, slug]))
+        return HttpResponseRedirect(reverse('sentry-manage-project-plugins', args=[project.team.slug, project.slug]))
 
-    if not plugin.is_enabled(project) or not plugin.can_enable_for_projects():
-        return HttpResponseRedirect(reverse('sentry-configure-project-plugin', args=[project.team.slug, project.slug, slug]))
+    redirect_to = reverse('sentry-configure-project-plugin', args=[project.team.slug, project.slug, slug])
+
+    if not (plugin.can_disable and plugin.is_enabled(project) and plugin.can_enable_for_projects()):
+        return HttpResponseRedirect(redirect_to)
 
     result = plugins.first('has_perm', request.user, 'configure_project_plugin', project, plugin)
     if result is False and not request.user.is_superuser:
@@ -446,4 +391,4 @@ def disable_project_plugin(request, team, project, slug):
 
     plugin.set_option('enabled', False, project)
 
-    return HttpResponseRedirect(reverse('sentry-configure-project-plugin', args=[project.team.slug, project.slug, slug]))
+    return HttpResponseRedirect(redirect_to)
